/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "function_classify_button.h"
#include "src/UtilityFunction/utility.h"
#include <QDebug>
#include <syslog.h>
#include <QStylePainter>
#include <QStyleOptionButton>

FunctionClassifyButton::FunctionClassifyButton(int width,
        int height,
        int iconSize,
        QString category,
        bool fullscreen,
        bool enabled,
        QWidget *parent):
    QPushButton(parent),
    m_width(width),
    m_height(height),
    m_iconSize(iconSize),
    m_category(category),
    m_fullscreen(fullscreen),
    m_enabled(enabled)
{
    this->setFlat(true);
    this->setFixedSize(m_width, m_height);
    this->setFocusPolicy(Qt::NoFocus);

    if (m_fullscreen) {
        updateIconState(Normal);
        this->setCheckable(true);
    } else {
        if (m_enabled) {
            updateIconState(Enabled);
        } else {
            updateIconState(Disabled);
        }

        this->setCheckable(false);
    }

    connect(this, &FunctionClassifyButton::toggled, this, &FunctionClassifyButton::reactToToggle);
    connect(this, &FunctionClassifyButton::clicked, this, &FunctionClassifyButton::buttonClickedSlot);
}

void FunctionClassifyButton::paintEvent(QPaintEvent *e)
{
    Q_UNUSED(e);
    QStylePainter painter(this);
    QStyleOptionButton option;
    initStyleOption(&option);

    if ((option.state & QStyle::State_Enabled) && (option.state & QStyle::State_MouseOver)) {
        painter.save();
        painter.setPen(Qt::NoPen);

        if (!m_fullscreen) {
            QColor color;

            if (option.state & QStyle::State_Selected) {
                if (g_curStyle == "ukui-dark") {
                    color.setNamedColor("#33FFFFFF");
                } else {
                    color.setNamedColor("#D1FFFFFF");
                }
            } else {
                if (g_curStyle == "ukui-dark") {
                    color.setNamedColor("#1AFFFFFF");
                } else {
                    color.setNamedColor("#8CFFFFFF");
                }
            }

            painter.setBrush(color);
        } else {
            painter.setOpacity(0.15);
            painter.setBrush(Qt::white);
        }

        painter.drawRoundedRect(option.rect, 4, 4);
        painter.restore();
    }

    if (!(option.state & QStyle::State_Enabled)) {
        painter.save();
        QColor color;
        if (!m_fullscreen) {
            if ( g_curStyle == "ukui-light") {
               color = Qt::black;
            } else {
                color = Qt::white;
            }
            color.setAlphaF(0.2);
        }
        painter.setPen(color);
        painter.drawText(option.rect, Qt::AlignCenter, m_category);
        painter.restore();
    }

    if (option.state & QStyle::State_Enabled) {
        painter.save();
        QColor color;

        if (m_fullscreen) {
            color = Qt::white;
        } else {
            if ( g_curStyle == "ukui-light") {
               color = Qt::black;
            } else {
               color = Qt::white;
            }
        }

        color.setAlphaF(0.9);
        painter.setPen(color);
        painter.drawText(option.rect, Qt::AlignCenter, m_category);
        painter.restore();
    }

    if (m_fullscreen && (option.state & QStyle::State_On)) {
        painter.save();
        painter.setPen(Qt::NoPen);
        QColor color = option.palette.color(QPalette::Text);
        //        color.setAlphaF(0.15);
        painter.setBrush(color);
        painter.setOpacity(0.15);
        painter.setBrush(Qt::white);
        painter.drawRoundedRect(option.rect, 4, 4);
        painter.restore();
    }

    QStyleOptionButton subopt = option;
    subopt.rect = painter.style()->subElementRect(QStyle::SE_PushButtonContents, &option, this);
    subopt.palette.setBrush(QPalette::HighlightedText, subopt.palette.text());
    painter.style()->drawControl(QStyle::CE_PushButtonLabel, &subopt, &painter, this);
    return;
}

void FunctionClassifyButton::updateBtnState()
{
    if (m_enabled) {
        updateIconState(Enabled);
    } else {
        updateIconState(Disabled);
    }
}

void FunctionClassifyButton::enterEvent(QEvent *e)
{
    Q_UNUSED(e);

    if (m_enabled) {
        updateIconState(Checked);
    }
}

void FunctionClassifyButton::leaveEvent(QEvent *e)
{
    Q_UNUSED(e);

    if (m_fullscreen) {
        if (!isChecked()) {
            updateIconState(Normal);
        }
    } else {
        if (m_enabled) {
            updateIconState(Enabled);
        }
    }
}

void FunctionClassifyButton::reactToToggle(bool checked)
{
    if (m_fullscreen) {
        if (checked) {
            updateIconState(Checked);
        } else {
            updateIconState(Normal);
        }
    }
}

void FunctionClassifyButton::buttonClickedSlot()
{
    Q_EMIT buttonClicked();
}

void FunctionClassifyButton::updateIconState()
{
    this->setFlat(true);
    QString picState;

    switch (m_state) {
        case Enabled:
            picState = "Enabled";
            break;

        case Disabled:
            picState = "Disabled";
            break;

        case Normal:
            picState = "Normal";
            break;

        case Checked:
            picState = "Checked";
            break;

        default:
            break;
    }

//    const auto ratio = devicePixelRatioF();
//    QPixmap pixmap = loadSvg(QString(":/data/img/mainviewwidget/%1-%2.svg").arg(m_category).arg(picState), m_iconSize * ratio);
//    QGSettings gsetting(QString("org.ukui.style").toLocal8Bit());
//    if (gsetting.get("style-name").toString() == "ukui-light") { //反黑
//        pixmap = drawSymbolicBlackColoredPixmap(pixmap);
//    } else {
//        pixmap = drawSymbolicColoredPixmap(pixmap); //反白
//    }
//    pixmap.setDevicePixelRatio(qApp->devicePixelRatio());
//    m_iconLabel->setPixmap(pixmap);
}

void FunctionClassifyButton::updateIconState(const FunctionClassifyButton::State state)
{
    if (state == m_state) {
        return;
    }

    m_state = state;
    QString picState;

    switch (state) {
        case Enabled:
            picState = "Enabled";
            break;

        case Disabled:
            picState = "Disabled";
            break;

        case Normal:
            picState = "Normal";
            break;

        case Checked:
            picState = "Checked";
            break;

        default:
            break;
    }

//    const auto ratio = devicePixelRatioF();
//    QPixmap pixmap = loadSvg(QString(":/data/img/mainviewwidget/%1-%2.svg").arg(m_category).arg(picState), m_iconSize * ratio);
//    if (!m_fullscreen) {
//        QGSettings gsetting(QString("org.ukui.style").toLocal8Bit());
//        if (gsetting.get("style-name").toString() == "ukui-light") { //反黑
//            pixmap = drawSymbolicBlackColoredPixmap(pixmap);
//        } else {
//            pixmap = drawSymbolicColoredPixmap(pixmap); //反白
//        }
//    } else {
//        pixmap = drawSymbolicColoredPixmap(pixmap);    //反白
//    }
//    pixmap.setDevicePixelRatio(qApp->devicePixelRatio());
//    m_iconLabel->setPixmap(pixmap);
//    updateTextState(state);
}


